
/* 
 * LCAUView.m -- Implementation file for the LCAUView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "LCAUView.h"
#import "drawingFuncs.h"
#import "nxlcau26.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>

extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

int    drawEvol=NO;
int    drawLine=NO;
int    drawMosaic=NO;
int    drawPlane=NO;
int    drawShort=NO;
int    drawThird=NO;

int    drawview=0;

int    repPlEv;

NXColor colst[KK];

extern char   arr1[], arr2[];
extern float  getNumber(NXStream *stream);
extern int    getSeparator(NXStream *stream);

extern int    co, dn;				/* connectivity, dimension */

void shortevolve(int m, int n);
void asctobin();
int  ix2(int i, int j);
int  ix3(int i, int j, int k);
int  ix4(int i, int j, int k, int l);
int  ix5(int i, int j, int k, int l, int m);
void onegen(int n);
void renewarry();
void savearry();
void wrru(char *c,
	  char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK]);   

#define MAXNUMLENGTH 50
#define DEFAULTRADIUS 1.0

@implementation LCAUView

void videocursor(p,i,j) int p, i, j; {}
void videoputc(c,i) char c; int i; {}
void videomode(i) int i; {}
void videopalette(i) int i; {}
void videoscroll(i,j,k,l,d,c) int i, j, k, l, d, c; {}
void videocattr(p,c,n,l) int p, c, n, l; {}


void videodot(i,j,l) int i, j, l; {

       if (l<2) NXSetColor(colst[l]);
  else if (l==2) NXSetColor(NX_COLORBLUE);
  else if (l==3) NXSetColor(NX_COLORBLACK);

    switch (drawview) {
      case 0: PSrectfill(2.0*(float)j,398.0-2.0*(float)i,1.25,1.25); break;
      case 1: PSrectfill(2.0*(float)j,200.0-2.0*(float)i,1.25,1.25); break;
      case 2: PSrectfill(8.0*(float)j+1.5,60.0-8.0*(float)i,4.75,4.75); break;
      case 3: PSrectfill(4.0*(float)j+2.0,48.0-4.0*(float)i,3.0,3.0); break;
      case 4: PSrectfill(2.0*(float)j,100.0-2.0*(float)i,1.25,1.25); break;
      case 5: PSrectfill(2.0*(float)j,2.0*(float)i,1.5,1.5); break;
      case 6: PSrectfill(10.0*(float)j,10.0*(float)i,9.0,9.0); break;
      case 7: PSrectfill(4.0*(float)j,4.0*(float)i,3.125,3.125); break;
      case 8: PSrectfill(150.0+(float)j,260.0-(float)i,1.0,1.0); break;
      case 9: PSrectfill(2.0*(float)j,173.0-2.0*(float)i,1.5,1.5); break;
      case 10: PSrectfill((float)j,(float)i,3.125,3.125); break;
      case 11: PSrectfill((float)j,(float)i,1.0,1.0); break;
      case 12: PSrectfill(2.0*(float)j,100.0-2.0*(float)i,1.25,1.25); break;
      case 13: PSrectfill(2.0*(float)j,151.0-2.0*(float)i,1.25,1.25); break;
      case 14: PSrectfill(2.0*(float)j,251.0-2.0*(float)i,1.25,1.25); break;
      default: break;}
}

/* m = line length; n = number of lines */

void shortevolve(m,n) int m, n; {int i, j, o;
  savearry();
  asctobin();
  o=YL/2-m/2;
  for (j=0; j<n; j++) {
    for (i=0; i<m; i++) videodot(j,i,arr1[i+o]);
    onegen(YL);
    }
  renewarry();
}

void drawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]);
}

void drawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]);
}

void manysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]);
}

void drawgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

/* --------------------------- E V O L U T I O N --------------------------- */

/* evWindow decides which evolutionary window to display, based */
/* on the dimension of the automaton, which in turn depends on  */
/* how the neighborhood's volume can be factorized		*/

- evWindow:sender {  switch (dn) {
     case 1:  [ev1Window makeKeyAndOrderFront: sender];
	      [ev2Window performClose: sender];
	      [ev3Window performClose: sender];
	      break;
     case 2:  [ev2Window makeKeyAndOrderFront: sender];
	      [ev1Window performClose: sender];
	      [ev3Window performClose: sender];
	      break;
     default: [ev3Window makeKeyAndOrderFront: sender];
	      [ev1Window performClose: sender];
	      [ev2Window performClose: sender];
	      break;}
  return self;
}

- drawShEvol:sender {
    drawview=7;
    drawShort=YES;
    [self display];
    drawShort=NO;
return self;
}

- printShEvol:sender {
    drawview=7;
    drawShort=YES;
    [keyWindow smartPrintPSCode: self];
    drawShort=NO;
return self;
}

- drawThEvol:sender {
    [thEvolWindow makeKeyAndOrderFront: self];
    drawview=6;
    drawThird=YES;
    [thEvolWindow display];
    drawThird=NO;
return self;
}

- printThEvol:sender {
    drawview=6;
    drawThird=YES;
    [keyWindow smartPrintPSCode: self];
    drawThird=NO;
return self;
}

- printLCAU:sender {
    if (keyWindow==shEvolWindow)  [self printShEvol:  sender];
    if (keyWindow==thEvolWindow)  [self printThEvol:  sender];
    if (keyWindow==evolWindow)    [self printEvolu:   sender];
    if (keyWindow==plaidWindow)   [self printPlaid:   sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyLCAU:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printLCAU:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyShEvol:sender {
NXRect zbounds;
id     pBoard;
  drawview=7;
  drawShort=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theShEvolView getBounds: &zbounds];
  [theShEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawShort=NO;
return self;
}

- copyThEvol:sender {
NXRect zbounds;
id     pBoard;
  drawview=6;
  drawThird=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theThEvolView getBounds: &zbounds];
  [theThEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawThird=NO;
return self;
}

- copyEvolu:sender {
NXRect zbounds;
id     pBoard;
  drawview=7;
  drawShort=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theShEvolView getBounds: &zbounds];
  [theShEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawShort=NO;
return self;
}

- copyLCAU:sender {
    if (keyWindow==shEvolWindow)  [self copyShEvol: sender];
    if (keyWindow==thEvolWindow)  [self copyThEvol: sender];
    if (keyWindow==evolWindow)    [self copyEvolu:  sender];
    if (keyWindow==plaidWindow)   [self copyPlaid:  sender];
return self;
}

- drawEvolu:sender {
  drawview=0;
  drawEvol=YES;
  [self display];
  drawEvol=NO;
return self;
}

- printEvolu:sender {
  drawview=0;
  drawEvol=YES;
  [keyWindow smartPrintPSCode: self];
  drawEvol=NO;
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else
    for (i=0; i<repPlEv; i++) {
      [genEv setIntValue: i];
      onegen(AL);
      [self display];
      }
  [genEv setIntValue: repPlEv];
  drawPlane=NO;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawPlane=YES;
    [self clear: self];
    [wholePanel smartPrintPSCode: self];
    drawPlane=NO;

return self;
}

- drawMosaic:sender {
  drawview=11;
  drawMosaic=YES;
  [self display];
  drawMosaic=NO;
	needsClearing = YES;
return self;
}

- printMosaic:sender {
  drawview=11;
  drawMosaic=YES;
  [wholePanel smartPrintPSCode: self];
  drawMosaic=NO;
	needsClearing = YES;
return self;
}

/* -------------------------------- L C A U -------------------------------- */

 /* all programs moved to RulePanel.m */
 
/* - - - - - - - - - - - -   L I N E   E D I T O R   - - - - - - - -*/

/* all programs moved to LinePanel.m */

/* -------------------------------- A R D N -------------------------------- */

/* all programs moved to ArdnPanel.m */

/* -------------------------------- R I J N -------------------------------- */

/* all programs moved to LiulGuph.m */

/* -------------------------------- P R O B -------------------------------- */

/* -------------------------------- * * * * -------------------------------- */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new LCAUView object.  First, an initFrame: message is sent 
 * to super to initialize LCAUView as a View.  Next, the LCAUView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	int     i, j;
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	[self setRadius:DEFAULTRADIUS];
	points = [[Storage alloc] initCount:0 elementSize:sizeof(NXPoint) description:"{ff}"];
	crossCursor = [NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];
	/* 
	 * Normally, the next two message expressions would be combined, as:
	 * 		readOut = [[Cell alloc] initTextCell:""];
	 * but are here separated for clarity.
	 */
	readOut = [Cell alloc];
	[readOut initTextCell:"pointies"];
	[readOut setBezeled:YES];
	loadPSProcedures();


  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);


  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the LCAUView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the LCAUView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although LCAUView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             ch[WNML], c[WNML];
	int		 i, j, k, od;
        double           p, q, s, dp, pee, cue;
	NXPoint		 *aPoint;
	NXRect		 plotRect;
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context. This simple test is OK for a View that prints 
	 * a single page. If there were multiple pages, the procedures would 
	 * be loaded multiple times. Although this is inefficient, it causes 
	 * no error. Views that print multiple pages should load procedures 
	 * such as these in an endPrologue method (see the View spec sheet). 
	 */
	if (NXDrawingStatus != NX_DRAWING) 
		loadPSProcedures();

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

// drawAxes(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);


        if (drawEvol==YES) {
   wrru(c,ascrule);
   [ruleNumber setStringValue: c];
	  evolve(0);
	  }

        if (drawShort==YES) {
   	  [self getBounds:&plotRect];
	  i=(int)(plotRect.size.width/4.0);
	  j=(int)(plotRect.size.height/4.0);
  	  od=dn; dn=1; shortevolve(i,j); dn=od;
	}

        if (drawThird==YES) {
  	  [self getBounds:&plotRect];
	  i=(int)(plotRect.size.width/10.0);
	  j=(int)(plotRect.size.height/10.0);
  	  od=dn; dn=1; shortevolve(i,j); dn=od;
	  }

        if (drawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "Moore (2,1) Plane Evolution"];
	drawsquare();
	break;
     case 3: 
	[window setTitle: "3-D face centered cubic Evolution"];
	drawcube();
	drawgrid();
	break;
     case 5: 
	[window setTitle: "von Neumann (2,2) Plane Evolution"];
	drawsquare();
	break;
     default: [window setTitle: "LCAU24 Plane Evolution"];break;}
	}

        if (drawMosaic==YES) {
   wrru(ch,ascrule);
   [ruleNumber setStringValue: ch];
   switch (dn) {
     case 2: 
	[window setTitle: "Moore (2,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     case 5: 
	[window setTitle: "von Neumann (2,2) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     default: [window setTitle: "LCAU24 Plane Evolution"];break;}
	}

        if (drawLine==YES) {
        for (j=0; j<8; j++) for (i=0; i<40; i++) videodot(j,i,arr1[i+40*j]);
	}

	/* now take each point and draw it */
	PSsetgray(NX_BLACK);
	i = [points count];
	while (i--) {
		aPoint = (NXPoint *)[points elementAt:i];
		drawCircle(aPoint->x, aPoint->y, radius);
	} 
	return self;
}

- clear:sender
/*
 * Clears the LCAUView by emptying its Storage object of all points
 * and then redisplaying the LCAUView.  This action is taken only if
 * the LCAUView's state requires it.
 */
{
	if (needsClearing) {
		[points empty];
		[self display];
		needsClearing = NO;
	}
	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the LCAUView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==6||drawview==7) 
		{[self setDrawOrigin:0 :0]; return self;}
	else 
		[self setDrawOrigin: -floor(width/2): -floor(height/2)]; 
	return self;
}

- registerPoint:(NXPoint *)aPoint
/*
 * Adds a point to the list the LCAUView keeps in its Storage object. 
 */
{
	[points addElement:aPoint];
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the LCAUView.  The LCAUView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the LCAUView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the LCAUView, the LCAUView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect, cellRect;
   	char	buffer[100];

         double  x, y, ex, wy, eks, wye;
	 int k;

	[crossCursor set];
   	[self getBounds:&plotRect];
 	NXSetRect(&cellRect, plotRect.origin.x, plotRect.origin.y, 100.0, 20.0);

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {snod=k;} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
		sprintf(buffer, "(%d, %d)", (int)aPoint.x, (int)aPoint.y); 
  		[readOut setStringValue:buffer];
  		[readOut drawInside:&cellRect inView:self];
				PSsetgray(NX_BLACK);
//				PSmoveto(ex+12.0,wy);
//				PSarc(ex, wy, 12.0, 0.0, 360.0);
//				PSstroke();
//				cntr[k][0]=ex;
//				cntr[k][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	    	/* on mouse-up, register point, inform delegate, and clean up state */
			[readOut setStringValue:""];
  			[readOut drawInside:&cellRect inView:self];
			[window flushWindow];
			if (NXPointInRect(&aPoint, &plotRect)) {
				PSsetgray(NX_BLACK);
				drawCircle(aPoint.x, aPoint.y, radius);
				[window flushWindow];
				[self registerPoint:&aPoint];
				needsClearing = YES;
				if (delegate && [delegate respondsTo:@selector(PlotView:pointDidChange:)])
					[delegate PlotView:self pointDidChange:&aPoint];
			}
			looping = NO;
		}	
   	} while (looping && (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


//   shortRijn=YES;
//   [window display];
//   shortRijn=NO;


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- setRadius:(float)aFloat
/*
 * Sets the value for the radius of the LCAUView's points.
 */
{
	radius = aFloat;
	return self;
}


- (float)radius
/*
 * Returns the value for the radius of the LCAUView's points.
 */
{
	return radius;
}


- read:(NXTypedStream *)stream
/*
 * Unarchives the LCAUView.  Initializes four of the LCAUView's instance 
 * variables to the values stored in the stream. 
 */
{
	[super read:stream];
	delegate = NXReadObject(stream);
	NXReadTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- write:(NXTypedStream *)stream
/*
 * Archives the LCAUView by writing its important instance variables to the 
 * stream. 
 */
{
	[super write:stream];
	NXWriteObjectReference(stream, delegate);
	NXWriteTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- awake
/*
 * Finishes initializing a LCAUView that has just been unarchived (see the read: method).
 */
{
	loadPSProcedures();
	return [super awake];
}


- (const char *)inspectorName
{
    return "LCAUViewInspector";
}

- plot:sender
/*
 * Responds to a plot: message by asking the LCAUView's delegate for a stream
 * containing the points to plot.  It then extracts number pairs from the 
 * stream and creates NXPoint structures with them.  For each structure it 
 * creates, the LCAUView sends itself a registerPoint: message to add the 
 * point to its list of points.  This simple parser ignores input it doesn't 
 * understand.
 */
{
	int	 	 c, sign, retval;
	float	 	 aNum;
	NXPoint	 	 aPoint;
	NXStream	*stream;
	BOOL		 processedLine = NO, gotFirst = NO, gotSecond = NO;
	
	if (delegate && [delegate respondsTo:@selector(PlotView:providePoints:)])
		[delegate PlotView:self providePoints:&stream];
	NXSeek(stream, 0, NX_FROMSTART);
	while ((c = NXGetc(stream)) != EOF){
		sign = 1;
		retval = getSeparator(stream);
		if (retval == 1) {
			c = NXGetc(stream);
		} else if (retval == -1) {
			break;
		}
		if (!NXIsDigit(c)) {
			if ((c == '-') && NXIsDigit(c = NXGetc(stream))){
					sign = -1;
			} else {
				while( (c != '\n') && (c != EOF)) {
					c = NXGetc(stream);
					processedLine = YES;
				}
			}
		}
		if (c == EOF)
			break;
		else if (processedLine) {
			processedLine = NO;
			continue;
		}
		aNum = getNumber(stream);
		if (!gotFirst) {
			aPoint.x = sign * aNum;
			gotFirst = YES;
		} else if (!gotSecond) {
			aPoint.y = sign * aNum;
			[self registerPoint:&aPoint];
			gotFirst = gotSecond = NO;
		} 
	}
	[self display];
	needsClearing = YES;
	return self;
}

int getSeparator(NXStream *stream)
/* 
 * Removes white space, commas, and plus signs from between 
 * number pairs.
 */
{
	int c, firstChar;
	
	NXUngetc(stream);
	c = firstChar = NXGetc(stream);
	while (NXIsSpace(c) || (c == ',') || (c == '+'))
		c = NXGetc(stream);
	/* 1 = ate something; 0 = ate nothing; -1 means EOF */
	if (c == firstChar)
		return 0;
	else if (c == EOF)
		return -1;
	NXUngetc(stream);
	return 1;
}

float getNumber(NXStream *stream)
/* 
 * Composes a floating point number from a string of number  
 * characters.
 */
{
	int c, i = 0;
	char temp[MAXNUMLENGTH];
	
	NXUngetc(stream);
	c = NXGetc(stream);
	while ((NXIsDigit(c) || (c == '.')) && (c != '\n') && (c != EOF)) {
		temp[i++] = c;
		c = NXGetc(stream);
	}
	NXUngetc(stream);
	temp[i] = 0;
	return (atof(temp));
}

@end

/* end LCAUView.m */
