
/* 
 * LiulGuph.m -- Implementation file for the LiulGuph class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import  "LiulGuph.h"
#import  "drawingFuncs.h"
#import  "nxlcau26.h"
// #import  "nxardn26.h"
// #import  "nxrijn26.h"
#import  <math.h> 
#import  <dpsclient/psops.h>
#import  <dpsclient/wraps.h>
#import  <stdlib.h>
#import  <string.h>
#include <stdio.h>
#include <time.h>

int    cnstRijn=NO;	/* YES/NO for evolve to constant/shift		*/
int    drawBasic=NO;	/* colored drawing of de Bruijn diagram itself  */
int    justRijn=NO;	/* black de Bruijn diagral of appropriate order */
int    lastDraw=0;	/* 0 = none, 1 = rijn, 2 = basic, 3 = generic   */
int    rijnBack=YES;	/* include an opaque background in the drawing  */
int    shortRijn=NO;	/* elaborate diagram whichever cnstRijn setting */


extern int    drawShort;
extern int    drawview;

int    rijnPerm;
int    rijnShift;
int    rijnGen;


extern int    co, dn;		/* connectivity, dimension */

extern NXColor colst[KK];

extern int    dbno;		/* node number in de Bruijn diagram 	*/
extern int    dd;		/* circumference of de Bruijn ring	*/
extern double dcntr[][2];

extern char   ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

void   asctobin();
int    closeto(double x, double y);
void   edijn(int g, int s, int p, int r);
void   finderset(int *e, twoparn *t);
void   howdown(int *sud);
int    level (int n);
int    prnins(int i, twoparn *z);
int    prouts(int i, twoparn *z);
void   prxtrmd(int *min, int *max);
void   prxtrml(int *min, int *max);
int    putlevel(twoparn *zmat, twoparn *xmat, int n);
int    shflev(int g, int s);
void   shortevolve(int m, int n);
void   upthere(int *sud);
void   wrru(char *c,
	    char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK]);


@implementation LiulGuph


/* -------------------------------- R I J N -------------------------------- */

- loadRijn:sender {
  if (rijnWindow==nil) {
    if (![NXApp loadNibSection: "rijn.nib" owner: self withNames: NO]) 
	{return nil;}
    lastDraw=0;
    [self rijnReset: self];
    }
  [rijnWindow makeKeyAndOrderFront: nil];
  return self;
  }

- rijn:sender {int i;
  rijnGen=[sender selectedRow];
  rijnShift = 5-(int)[sender selectedCol];
  i=shflev(rijnGen,rijnShift);
  [self rijnHowBig: self];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
  rijnPerm=[whichPermutation selectedRow];
  edijn(rijnGen,5-rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=NO;
return self;
}

- rijnConst:sender {
  rijnGen=0;
  rijnShift=[sender selectedCol];
  [self rijnHowBig: self];
  [whichShift setEmptySelectionEnabled: YES];
  [whichShift selectCellAt: -1: 0];
  rijnPerm=[whichPermutation selectedRow];
  edijn('A'-'a'+rijnGen,rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=YES;
return self;
}

- rijnReset:sender {
  [whichShift setEmptySelectionEnabled: YES];
  [whichShift selectCellAt: -1: 0];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
return self;
}

/* dd = k^(2r)*k^(2*level) */
/* the code for single generation deBruijn diagrams has been simplified */
- rijnHowBig:sender {int i; double th, dt;
    switch(0) {
      case 0:  dd=4096; break;
      default: dd=4096; break;}
  dt=6.28318/((double)(dd));
  for (i=0, th=0.5*dt; i<dd; i++, th+=dt) {
    dcntr[i][0]=200.0-180.0*sin(th);
    dcntr[i][1]=200.0-180.0*cos(th);
    }
return self;
}

/* load a panel for selecting the type of de Bruijn diagram */

- rijnRanSam:sender {int  i;
  i='z';
  if (strcmp([[sender selectedCell] title],"random sample")==0) i='r';
  if (strcmp([[sender selectedCell] title],"selected node")==0) i='n';
  if (strcmp([[sender selectedCell] title],"scan links")==0)    i='i';
  if ([dbBranches state]) {return self;}
  if (cnstRijn) edshc(rijnGen,i); else edshf(rijnGen,rijnShift,i);
  [evWindow makeKeyAndOrderFront: sender];
  [evButton performClick: sender];
return self;
}

- basicRijn:sender {char c[WNML], d[37+WNML];
  drawBasic=YES;
  [self rijnReset: self];
//  wru(c);
//  sprintf(d,"lcau26 rule  %s  de Bruijn diagram",c);
  [drawPanel makeKeyAndOrderFront:self];
  [drawPanel setTitle: d];
  [drawPanel display];
  drawBasic=NO;
  lastDraw = 2;
return self;
}

- fullRijn:sender {char c[WNML], d[37+WNML];
    shortRijn=YES;
//    wru(c);
//    sprintf(d,"lcau26 rule  %s  de Bruijn diagram",c);
    [drawPanel makeKeyAndOrderFront:self];
    [drawPanel setTitle: d];
    [drawPanel display];
    shortRijn=NO;
    lastDraw = 1;
return self;
}

- shewRijn:sender {char d[37];
    justRijn=YES;
    sprintf(d,"lcau26 generic de Bruijn diagram");
    [drawPanel makeKeyAndOrderFront:self];
    [drawPanel setTitle: d];
    [drawPanel display];
    justRijn=NO;
    lastDraw = 3;
return self;
}

- printGUPH:sender {
    switch (lastDraw) {
      case 0: break;
      case 1: shortRijn=YES; break;
      case 2: drawBasic=YES; break;
      case 3: justRijn=YES;  break;
      default: break;}
    [drawPanel smartPrintPSCode: self];
    switch (lastDraw) {
      case 0: break;
      case 1: shortRijn=NO; break;
      case 2: drawBasic=NO; break;
      case 3: justRijn=NO;  break;
      default: break;}
return self;
}

- windowDidBecomeKey:sender {
	[rijnWindow orderFront: self];
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyGUPH:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printGUPH:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [drawTheView getBounds: &zbounds];
  [drawTheView writePSCodeInside: &zbounds to: pBoard]; 
  *flag = NO;
return self;
}

- copyGUPH:sender {
    rijnBack=NO;
    switch (lastDraw) {
      case 0: break;
      case 1: [self copyWhatsit: &shortRijn]; break;
      case 2: [self copyWhatsit: &drawBasic]; break;
      case 3: [self copyWhatsit: &justRijn];  break;
      default: break;}
    rijnBack=YES;
return self;
}

/* -------------------------------- C Y C L -------------------------------- */

/* the cycle programs have been moved to cycle.nib and associated files */

/* -------------------------------- P A I R -------------------------------- */

/* the pair programs have been moved to pair.nib and associated files */
/* even so, they require too much space and time to be used in lcau26 */

/* -----------------------------  S U B S E T  ---------------------------- */

/* the subset programs have been moved to suset.nib and associated files  */
/* even so, they require far too much space and time to be used in lcau26 */


/* - - - - - - - - - - - - -    L I U L G U P H   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new LiulGuph object.  First, an initFrame: message is sent 
 * to super to initialize LiulGuph as a View.  Next, the LiulGuph sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
//    NXPoint	spot;
	
    [super initFrame: frameRect];
//    [self translate: floor(frame.size.width/2): floor(frame.size.height/2)];
    [self setOpaque:YES];
//    crossCursor = 
//	[NXCursor newFromImage: [NXImage newFromSection: "circle.tiff"]];
//    spot.x = spot.y = 7.0;
//    [crossCursor setHotSpot:&spot];

    [self setFlipped: YES];

    return self;

printf("initFrame from LiulGuph\n");
}


- setDelegate:anObject
/*
 * Sets the LiulGuph's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the LiulGuph's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although LiulGuph only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char            c[WNML], timday[30];
	int	 	i, j, k;
        double          p, q, s;
  struct tm  *timeptr;
  time_t  secsnow;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (rijnBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);

/* for the 1-D de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - - */

        if (drawBasic==YES) {
		PSsetgray(NX_BLACK); 
   [[Font newFont: "Times-Italic" size: 18.0 matrix:NX_FLIPPEDMATRIX] set];
		for (i=0; i<KK; i++) sijn(i,i); 
		PSstroke();
		}

        if (justRijn==YES) {
		PSsetgray(NX_BLACK); 
		ijn(dd,KK,1); 
		PSstroke();
   [timeDate setStringValue: timday];
		}

        if (shortRijn==YES) {
  	[[Font newFont: "Courier" size: 20.0 matrix:NX_FLIPPEDMATRIX] set];
   wrru(c,ascrule);
   [ruleNumber setStringValue: c];
   PSmoveto(290.0,14.0);
   PSshow(c);
   PSmoveto(12.0,14.0);
  	[[Font newFont: "Courier" size: 12.0 matrix:NX_FLIPPEDMATRIX] set];
   if (cnstRijn) sprintf(c,"evolving to %1d in %1d gen",rijnShift,rijnGen+1); 
	else {if (rijnPerm==0) 
	  sprintf(c,"shift %1d in %1d gen",-rijnShift,rijnGen+1);
	  else 
sprintf(c,"shift %1d in %1d gen perm %d",-rijnShift,rijnGen+1,rijnPerm);}
   PSshow(c);
  	[[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
		PSsetgray(NX_BLACK); 
		if (cnstRijn) edcst(rijnGen); else edshg(rijnGen,rijnShift);
		PSstroke();

   [timeDate setStringValue: timday];

		}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the LiulGuph is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
// 	[self setFlipped: YES];
	[super sizeTo:width :height]; 
	if (drawview==4||drawview==7) 
		{[self setDrawOrigin:0 :0]; return self;}
	else 
		[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
   [self display];
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the LiulGuph.  The LiulGuph changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the LiulGuph updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the LiulGuph, the LiulGuph 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;

         double  x, y, ex, wy, eks, wye;
	 int i, j, k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the 1-d de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - */

 			[window setDocEdited:YES];
		        aPoint = theEvent->location;
		        [self convertPoint:&aPoint fromView:nil];
                        ex=aPoint.x;
                        wy=aPoint.y;
			dbno=nearest(ex,wy);
			[nodeNumber setIntValue: dbno];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {
		eks=0.05*(ex-200.0);
		wye=0.05*(wy-200.0);
				dcntr[dbno][0]=ex-eks;
				dcntr[dbno][1]=wy-wye;
		} else {
	[self lockFocus];
         PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
			PSsetgray(NX_BLACK);
			PSmoveto(ex+12.0,wy);
			PSarc(ex, wy, 12.0, 0.0, 360.0);
			PSstroke();
			dcntr[dbno][0]=ex;
			dcntr[dbno][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent= 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];
	}

   shortRijn=YES;
   [self display];
   shortRijn=NO;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end LiulGuph.m */
