
/* 
 * VN2DView.m -- Implementation file for the VN2DView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "VN2DView.h"
#import "drawingFuncs.h"
#import "nxlcau26.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12				/* maximum binomial coefficient */
#define NG 10				/* number of Golay Classes	*/
#define GH ((DS)/KK)		/* DS/KK= half number of neighborhoods	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char ttrule[];			/* evolutionary rule */
char    sqrule22[DS];	/* evolutionary rule       */
char    qarule22[DS];	/* auxiliary rule flags    */

extern char    arr1[], arr2[];
       int             brr2[AL];
extern double  bp[];			/* Bernstein coefficients    */
extern int     gc[];			/* Golay code key array      */
extern int     tc[];			/* totalistic key array      */
int     llf22[KK];			/* save number of live cells */
int	lf22[KK];	/* number of live cells	   */
int     stus[KK*(TS-1)];/* s-totalistic use stats  */
int     gus22[KK*NG];	/* golay use statistics    */
int     rus22[DS];	/* rule use statistics     */
int     ttus[TS];	/* totalistic use stats    */
int     gc22[DS];	/* Golay code key array    */
int     stc22[DS];	/* s-totalistic key array  */

extern NXColor colst[KK];    		/* Array for state colors    */

int    cheqqBoard=NO;
int    drawwARBack=YES;
int    drawwBnstein=NO;
int    drawwFreqs=NO;
int    drawwQuency=NO;
int    drawwRturn=NO;
int    drawwSmall=NO;
int    drawwSQuency=NO;
int    drawwUQuency=NO;
int    drawwTrajec=NO;
int    slowwFreqs=NO;
int    smaulSmall=NO;
int    smaulMosaic=NO;

int    lasttPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lasttTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    motTiles=3;
int    pDotts=NO;
int    skippThree=NO;
int    skippSThree=NO;
int    skippUThree=NO;
int    strettchU=125;
int    trajjLen=10;
int    reppPlEv=0;
int    rturrnNum=1;

int     tottpmut[TS] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13};
int     stottpmut[KK*(TS-1)] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13};
int     golppmut[KK*NG] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19};
int     gtbb[NG] = {0,1,2,2,3,3,4,4,5,6};
double  ttbb[TS] = {1.0,7.0,21.0,35.0,35.0,21.0,7.0,1.0};
NXColor gollcolst[KK*NG];    		/* Array for surround colors     */
NXColor tottcolst[TS];    		/* Array for totalistic colors   */
NXColor stottcolst[KK*(TS-1)];    	/* Array for s-totalistic colors */

extern int    drawview;
extern int    co, dn;

void    arbdot(int i, int j, int l);
void    arbline(double x, double y, int l);
void    argfram(int n, int l);
void    argdiag(int n, int l);
void    arronegen(char *aa, int *bb, int m, int n); 
void    arrscreen(char *arr1, int i, int j);
void    asctobin();
void	asctoqar22();
void	asczero();
void    barrgraf(int m, int n);
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
void    mannyrect(int i, int j, int m, int n);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    smaulgrid(int m, int n);
void    arrstatecount(char *aa, int m, int n);
void    sujjrule(int i);
int     totonu(int n);
void    videodot(int row, int col, int color);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    wxsscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation VN2DView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadSqar:sender {
  if (SqarWindow==nil) {
    if (![NXApp loadNibSection: "vn2d.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=2;
  [SqarWindow makeKeyAndOrderFront: sender];
  [sqarWindow orderFront: nil];
  [self golaySurround: sender];
  [self copyRule: sender];
return self;
}

void drawwsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void mannysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawwgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQAR:sender {
   if (keyWindow==SqarWindow) 
	{[SqarWindow smartPrintPSCode: self];}
   else if (keyWindow==GolayPanel || keyWindow==stretchWindow) 
	{[keyWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lasttPlane) {
      case 0: break;
      case 3: drawwSmall=YES;  break;
      case 4: smaulMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lasttPlane) {
      case 0: break;
      case 3: drawwSmall=NO;  break;
      case 4: smaulMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQAR:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQAR:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqarView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqarView getBounds: &zbounds];
  [SqarView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyGolayPanel:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [GolayView getBounds: &zbounds];
  [GolayView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyQuency:sender {
NXRect zbounds;
id     pBoard;
  drawwQuency = YES;
  drawview = 14;
  drawwARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheView state]) {
    [freqView getBounds: &zbounds];
    [freqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[freqWindow contentView] getBounds: &zbounds];
    [[freqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawwARBack = YES;
  drawwQuency = NO;
return self;
}

- copySQuency:sender {
NXRect zbounds;
id     pBoard;
  drawwSQuency = YES;
  drawview = 14;
  drawwARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheSView state]) {
    [sfreqView getBounds: &zbounds];
    [sfreqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[sfreqWindow contentView] getBounds: &zbounds];
    [[sfreqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawwARBack = YES;
  drawwSQuency = NO;
return self;
}

- copyUQuency:sender {
NXRect zbounds;
id     pBoard;
  drawview = 14;
  drawwUQuency = YES;
  drawwARBack = NO;
  if ([reStock state]) savearry();
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheUView state]) {
    [ufreqView getBounds: &zbounds];
    [ufreqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[ufreqWindow contentView] getBounds: &zbounds];
    [[ufreqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  if ([reStock state]) renewarry();
  drawwARBack = YES;
  drawwUQuency = NO;
return self;
}

- copyStretch:sender {
NXRect zbounds;
id     pBoard;
  drawview = 14;
  strettchU=375;
  drawwUQuency = YES;
  drawwARBack = NO;
  if ([reStock state]) savearry();
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
    [stretchView getBounds: &zbounds];
    [stretchView writePSCodeInside: &zbounds to: pBoard]; 
  if ([reStock state]) renewarry();
  drawwARBack = YES;
  drawwUQuency = NO;
  strettchU=125;
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawwARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawwARBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawwARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawwARBack=YES;
  *flag = NO;
return self;
}

- copySQAR:sender {
   if (keyWindow==SqarWindow) {[self copySqarView: self];}
   else if (keyWindow==GolayPanel) {[self copyGolayPanel: self];}
   else if (keyWindow==trajWindow) {
     savearry();
     switch (lasttTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawwTrajec]; break;
          case 2: [self copyWhoosit: &drawwRturn];  break;
          case 3: [self copyWhoosit: &drawwFreqs];  break;
          case 4: [self copyWhoosit: &slowwFreqs];  break;
          default: break;}
     renewarry();}
   else if (keyWindow==planeWindow) switch (lasttPlane) {
     case 0: break;
     case 3: drawview=7; reppPlEv=1; [self copyWhatsit: &drawwSmall];   break;
     case 4: savearry(); drawview=11; [self copyWhatsit: &smaulMosaic]; 
		renewarry(); break;
     default: break;}
   else if (keyWindow==freqWindow)    [self copyQuency: self];
   else if (keyWindow==sfreqWindow)   [self copySQuency: self];
   else if (keyWindow==ufreqWindow)   [self copyUQuency: self];
   else if (keyWindow==stretchWindow) [self copyStretch: self];
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smaulSmall=YES;
  [planeView display];
  smaulSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  cheqqBoard=YES;
  [planeView display];
  cheqqBoard=NO;
return self;
}

- uprule:sender {sujjrule(0); [self anyrule: self]; return self;}
- lerule:sender {sujjrule(1); [self anyrule: self]; return self;}
- dorule:sender {sujjrule(3); [self anyrule: self]; return self;}
- rirule:sender {sujjrule(2); [self anyrule: self]; return self;}
- idrule:sender {sujjrule(4); [self anyrule: self]; return self;}
- mjrule:sender {sujjrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void sujjrule(i) int i; {
int j, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
  for (i9=0; i9<KK; i9++)
  for (ia=0; ia<KK; ia++) 
  for (ib=0; ib<KK; ib++)
  for (ic=0; ic<KK; ic++) 
{
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=0; if (i0+i1+i2+i3+i4+i5+i6+i7+i8+i9+ia+ib+ic>7) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawwSmall=YES;
    asctoqar22();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&reppPlEv);
    [self display];
    if (reppPlEv>0) for (k=0; k<reppPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawwSmall=NO;
    lasttPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawwSmall=YES;
    reppPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawwSmall=NO;
return self;
}

- drawSmMosaic:sender {
    motTiles=[moTies selectedRow]+2;
    drawview=11;
    smaulMosaic=YES;
    asctoqar22();
    savearry();
    [self display];
    renewarry();
    smaulMosaic=NO;
    lasttPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smaulMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smaulMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajjLen);
    pDotts=[probDots state];
    drawview=13;
    drawwTrajec=YES;
    asctoqar22();
    savearry();
    [trajView display];
    renewarry();
    drawwTrajec=NO;
    lasttTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    drawwTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturrnNum);
    drawview=13;
    drawwRturn=YES;
    asctoqar22();
    savearry();
    [trajView display];
    renewarry();
    drawwRturn=NO;
    lasttTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawwRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwRturn=NO;
return self;
}

- drawFreq:sender {
    drawview=13;
    drawwFreqs=YES;
    asctoqar22();
    savearry();
    [trajView display];
    renewarry();
    drawwFreqs=NO;
    lasttTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawwFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwFreqs=NO;
return self;
}

- slowFreq:sender {
    drawview=13;
    slowwFreqs=YES;
    asctoqar22();
    savearry();
    [trajView display];
    renewarry();
    slowwFreqs=NO;
    lasttTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowwFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowwFreqs=NO;
return self;
}

- drawFrequency:sender {
    skippThree=[eachAndAll state];
    tottpmut[0]=[freqPerm0 selectedCol];
    tottpmut[1]=[freqPerm1 selectedCol];
    tottpmut[2]=[freqPerm2 selectedCol];
    tottpmut[3]=[freqPerm3 selectedCol];
    tottpmut[4]=[freqPerm4 selectedCol];
    tottpmut[5]=[freqPerm5 selectedCol];
    tottpmut[6]=[freqPerm6 selectedCol];
    tottpmut[7]=[freqPerm7 selectedCol];
    tottcolst[0]=[freqColor0 color];
    tottcolst[1]=[freqColor1 color];
    tottcolst[2]=[freqColor2 color];
    tottcolst[3]=[freqColor3 color];
    tottcolst[4]=[freqColor4 color];
    tottcolst[5]=[freqColor5 color];
    tottcolst[6]=[freqColor6 color];
    tottcolst[7]=[freqColor7 color];
    drawview=14;
    drawwQuency=YES;
    asctoqar22();
    savearry();
    [freqView display];
    renewarry();
    drawwQuency=NO;
return self;
}

- printFrequency:sender {
    drawview=14;
    drawwQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwQuency=NO;
return self;
}

- drawSemiFreq:sender {int i;
    skippSThree=[eachAndSAll state];
    for (i=0; i<TS-1; i++) {
	stottpmut[i]=[[sfreqPerm0 cellAt: i: 0] intValue];
	stottpmut[i+TS-1]=[[sfreqPerm1 cellAt: i: 0] intValue];
	}
    stottcolst[0]=[sfreqColor00 color];
    stottcolst[1]=[sfreqColor01 color];
    stottcolst[2]=[sfreqColor02 color];
    stottcolst[3]=[sfreqColor03 color];
    stottcolst[4]=[sfreqColor04 color];
    stottcolst[5]=[sfreqColor05 color];
    stottcolst[6]=[sfreqColor06 color];
    stottcolst[7]=[sfreqColor10 color];
    stottcolst[8]=[sfreqColor11 color];
    stottcolst[9]=[sfreqColor12 color];
    stottcolst[10]=[sfreqColor13 color];
    stottcolst[11]=[sfreqColor14 color];
    stottcolst[12]=[sfreqColor15 color];
    stottcolst[13]=[sfreqColor16 color];
    drawview=14;
    drawwSQuency=YES;
    asctoqar22();
    savearry();
    [sfreqView display];
    renewarry();
    drawwSQuency=NO;
return self;
}

- printSemiFreq:sender {
    drawview=14;
    drawwSQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwSQuency=NO;
return self;
}

- drawGolayFreq:sender {int i;
    skippUThree=[eachAndUAll state];
    for (i=0; i<NG; i++) {
	golppmut[i]=[[ufreqPerm0 cellAt: i: 0] intValue];
	golppmut[NG+i]=[[ufreqPerm1 cellAt: i: 0] intValue];
	}
    gollcolst[0]=[ufreqColor00 color];
    gollcolst[1]=[ufreqColor01 color];
    gollcolst[2]=[ufreqColor02 color];
    gollcolst[3]=[ufreqColor03 color];
    gollcolst[4]=[ufreqColor04 color];
    gollcolst[5]=[ufreqColor05 color];
    gollcolst[6]=[ufreqColor06 color];
    gollcolst[7]=[ufreqColor07 color];
    gollcolst[8]=[ufreqColor08 color];
    gollcolst[9]=[ufreqColor09 color];
    gollcolst[10]=[ufreqColor10 color];
    gollcolst[11]=[ufreqColor11 color];
    gollcolst[12]=[ufreqColor12 color];
    gollcolst[13]=[ufreqColor13 color];
    gollcolst[14]=[ufreqColor14 color];
    gollcolst[15]=[ufreqColor15 color];
    gollcolst[16]=[ufreqColor16 color];
    gollcolst[17]=[ufreqColor17 color];
    gollcolst[18]=[ufreqColor18 color];
    gollcolst[19]=[ufreqColor19 color];
    drawview=14;
    asctoqar22();
    if ([reStock state]) savearry();
    if ([stretchOut state]) {
	[stretchWindow makeKeyAndOrderFront: sender];
        drawwUQuency=YES;
	strettchU=375;
	[stretchView display];
	strettchU=125;
	} else {drawwUQuency=YES; [ufreqView display];}
    if ([reStock state]) renewarry();
    drawwUQuency=NO;
return self;
}

- printGolayFreq:sender {
    drawview=14;
    drawwUQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawwUQuency=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void mannyrect(i,j,m,n) int i, j, m, n; {int ii, jj;
  asctoqar22();
  for (ii=0; ii<m; ii++) for (jj=0; jj<n; jj++) 
	videodot(i*m+ii,j*n+jj,arr1[jj+n*ii]);
  arronegen(arr1,brr2,m,n);
}

/* m = high/motTiles, n = wide/motTiles */
void smaulgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)motTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<motTiles+1; i++) {
  PSmoveto((double)(n*i),0.0); PSlineto((double)(n*i),mo*em);}
  for (i=0; i<motTiles+1; i++) {
  PSmoveto(0.0,(double)(m*i)); PSlineto(mo*en,(double)(m*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawwBnstein=YES;
    [bernView display];
    drawwBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- selfBern:sender {
    drawview=12;
    drawwBnstein=YES;
    [self display];
    drawwBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- printRule:sender {
   drawview=12;
   drawwBnstein=YES;
   [keyWindow printPSCode:self];
   drawwBnstein=NO;
return self;
}

- golaySurround:sender {id cell; char *c; int i, j, ii;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
	ii=c[0]-'0'+KK*(c[1]-'0'+KK*(c[2]-'0'+KK*(c[3]-'0'+
		KK*(c[4]-'0'+KK*(c[5]-'0')))));
        gc[ii]=i; gc[DS/KK+ii]=NG+i;
	}
      }
return self;
}

- enterRule:sender {id cell; char *c, ch[WNML]; int i, j;
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

//     for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//       cell=[theZMatrix cellAt: j: i];
//       if ([cell isEnabled]) {
// 	c=[cell title];
//         ascrule[c[0]-'0']
// 		[c[1]-'0']
// 		[c[2]-'0']
// 		[c[3]-'0']
// 		[c[4]-'0']
// 		[c[5]-'0']
// 		[0]         =[cell intValue]+'0';
// 	}
//       cell=[theOMatrix cellAt: j: i];
//       if ([cell isEnabled]) {
// 	c=[cell title];
//         ascrule[c[0]-'0']
// 		[c[1]-'0']
// 		[c[2]-'0']
// 		[c[3]-'0']
// 		[c[4]-'0']
// 		[c[5]-'0']
// 		[1]         =[cell intValue]+'0';
// 	}
//       }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
      [bernView selfBern: nil];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {id cell; char *c, ch[WNML]; int i, j;
//     for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//       cell=[theZMatrix cellAt: j: i];
//       if ([cell isEnabled]) {
// 	c=[cell title];
//         [cell setIntValue: ascrule[c[0]-'0']
// 				[c[1]-'0']
// 				[c[2]-'0']
// 				[c[3]-'0']
// 				[c[4]-'0']
// 				[c[5]-'0']
// 				[0]-'0'        ];
// 	}
//       cell=[theOMatrix cellAt: j: i];
//       if ([cell isEnabled]) {
// 	c=[cell title];
//         [cell setIntValue: ascrule[c[0]-'0']
// 				[c[1]-'0']
// 				[c[2]-'0']
// 				[c[3]-'0']
// 				[c[4]-'0']
// 				[c[5]-'0']
// 				[1]-'0'        ];
// 	}
//       }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [bernView selfBern: nil];
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqarWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqarWindow setDocEdited: NO];
     [bernView selfBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new VN3DView object.  First, an initFrame: message is sent 
 * to super to initialize VN3DView as a View.  Next, the VN3DView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the VN3DView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the VN3DView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although VN3DView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, ii, jj;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawwARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawwBnstein==YES) {
//	  for (i=0; i<TS; i++) bp[i]=0.0;
//	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//		if ([[theZMatrix cellAt: j: i] state]) bp[gtbb[i]]+=1.0;
//		if ([[theOMatrix cellAt: j: i] state]) bp[gtbb[i]+1]+=1.0;
//		}
	berncoeff();
	  argfram(50,3);
	  argdiag(50,3);
	  barrgraf(TS-1,50);
	  }

	if (drawwTrajec==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  barrgraf(TS-1,75);
	  arrstatecount(arr1,64,64);
	  for (j=0; j<KK; j++) llf22[j]=lf22[j];
	  arronegen(arr1,brr2,64,64);
	  if (pDotts) arbdot((75*llf22[1])/4096,(75*lf22[1])/4096,3);
	    else arbline(0.0367*((double)lf22[1]),0.0367*((double)llf22[1]),0);
	  for (i=0; i<trajjLen; i++) {
	    for (j=0; j<KK; j++) llf22[j]=lf22[j];
	    arronegen(arr1,brr2,64,64);
	    if (pDotts) arbdot((75*llf22[1])/4096,(75*lf22[1])/4096,3);
	  else arbline(0.0367*((double)lf22[1]),0.0367*((double)llf22[1]),1);
	    }
	  PSstroke();
	  }

	if (drawwRturn==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  barrgraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturrnNum; j++) arronegen(arr1,brr2,64,64);
	    arbdot(i,(75*lf22[1])/4096,3);
	    }
	  }

	if (drawwFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i++) {
	    arronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (ttus[j]>57) for (l=0; l<ttus[j]; l+=58) arbdot(i,k++,j%3);
	    arbdot(i,(75*lf22[1])/4096,3);
	    }
// printf("teecees:"); for (i=0; i<DS; i++) printf("%5d ",tc[i]); printf("\n");
printf("tussies: "); for (i=0; i<TS; i++) printf("%5d ",ttus[i]); printf("\n");
	  }

	if (slowwFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i+=3) {
	    arronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (ttus[j]>57) for (l=0; l<ttus[j]; l+=58) {
		  arbdot(i,k,j%3);
		  arbdot(i+1,k,j%3);
		  arbdot(i+2,k,j%3);
		  k++;
		  }
	    arbdot(i,(75*lf22[1])/4096,3);
	    arbdot(i+1,(75*lf22[1])/4096,3);
	    arbdot(i+2,(75*lf22[1])/4096,3);
	    }
	  }

	if (drawwQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skippThree) ii=i/3+43*(i%3);
	    arronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++) {
		jj=tottpmut[j];
		NXSetColor(tottcolst[jj]);
		if (ttus[jj]>32) 
		    for (l=0; l<ttus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf22[1])/4096,3);
	    }
	  }

	if (drawwSQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skippSThree) ii=i/3+43*(i%3);
	    arronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<KK*(TS-1); j++) {
		jj=stottpmut[j];
		NXSetColor(stottcolst[jj]);
		if (stus[jj]>32) 
		    for (l=0; l<stus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf22[1])/4096,3);
	    }
printf("stottpmut: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",stottpmut[i]); 
printf("\n");
printf("stussies: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",stus[i]); 
printf("\n");
	  }

	if (drawwUQuency==YES) {
	  if (strettchU==125) argfram(125,3);
	  n=strettchU/3+2;
	  for (i=0; i<strettchU; i++) {
	    ii=i; if (skippUThree) ii=i/3+n*(i%3);
	    arronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<KK*NG; j++) {
		jj=golppmut[j];
		NXSetColor(gollcolst[jj]);
		if (gus22[jj]>32) 
		    for (l=0; l<gus22[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf22[1])/4096,3);
	    }
printf("golppmut: "); 
for (i=0; i<KK*NG; i++) printf("%5d ",golppmut[i]); 
printf("\n");
printf("gussies: "); 
for (i=0; i<KK*NG; i++) printf("%5d ",gus22[i]); 
printf("\n");
	  }

        if (cheqqBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)motTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smaulSmall==YES) {
	  mo=(double)motTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawwSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);       /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  if (j*i < AL) {
	    arrscreen(arr1,j,i);
	    if(reppPlEv>0) arronegen(arr1,brr2,j,i);
	    } else printf("volume = %6d, AL =  %6d\n",j*i,AL);
	  }

        if (smaulMosaic==YES) {
	  mo=(double)motTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  if (i*j < AL) {
	    for (k=0; k<motTiles; k++) for (l=0; l<motTiles; l++) {
	      mannyrect(k,l,i,j);
	      [window flushWindow];
	      }
	    smaulgrid(i,j);
	    } else printf("volume = %6d\n",i*j);
	  }

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the VN3DView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the VN3DView.  The VN3DView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the VN3DView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the VN3DView, the VN3DView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/*  - - - - - - - -  n x s q a r 2 2 . c  - - - - - - - - - - - - - - - - */

/* SQAR22.C					*/
/* Three-Dimensional Square Lattice 		*/
/* binary cells, second neighbors		*/
/* ---- von Neumann neighborhoods ----		*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 July 1988 - von Neumann neighborhgood	*/

void asctoqar22() {int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
  for (i9=0; i9<KK; i9++)
  for (ia=0; ia<KK; ia++)
  for (ib=0; ib<KK; ib++)
  for (ic=0; ic<KK; ic++)
    sqrule22[i0+KK*
		(i1+KK*
		(i2+KK*
		(i3+KK*
		(i4+KK*
		(i5+KK*
		(i6+KK*
		(i7+KK*
		(i8+KK*
		(i9+KK*
		(ia+KK*
		(ib+KK*
		ic)))))))))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic];
}

void qar22toasc() {int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
  for (i9=0; i9<KK; i9++)
  for (ia=0; ia<KK; ia++)
  for (ib=0; ib<KK; ib++)
  for (ic=0; ic<KK; ic++)
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic] = 
    	sqrule22[i0+KK*
		(i1+KK*
		(i2+KK*
		(i3+KK*
		(i4+KK*
		(i5+KK*
		(i6+KK*
		(i7+KK*
		(i8+KK*
		(i9+KK*
		(ia+KK*
		(ib+KK*
		ic)))))))))))]; 
}

/* 	     layout for cell indexing		*/
/*						*/
/*		 |				*/
/*		 0    				*/
/*	       1 8 7 				*/
/*      - -  2 9 c b 6 - -  	       		*/
/*	       3 a 5 			   	*/
/*	         4				*/
/*		 |				*/
/*						*/

/* calculate one generation of evolution */
/* aa[m][n] = original and final plane   */
/* bb[m][n] = intermediate scratch plane */
/* m = number of rows on the torus       */
/* n = number of columns on the torus    */
void arronegen(ar,br,m,n) char *ar; int *br; int m, n; {int i0, i1, gn;

/* count neighbors of center hypercube */

for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
	    gn=      ar[inx2(i0-2, i1,   m,n)];
	    gn=KK*gn+ar[inx2(i0-1, i1-1, m,n)];
	    gn=KK*gn+ar[inx2(i0,   i1-2, m,n)];
	    gn=KK*gn+ar[inx2(i0+1, i1-1, m,n)];
	    gn=KK*gn+ar[inx2(i0+2, i1,   m,n)];
	    gn=KK*gn+ar[inx2(i0+1, i1+1, m,n)];
	    gn=KK*gn+ar[inx2(i0,   i1+2, m,n)];
	    gn=KK*gn+ar[inx2(i0-1, i1+1, m,n)];
	    gn=KK*gn+ar[inx2(i0-1, i1,   m,n)];
	    gn=KK*gn+ar[inx2(i0,   i1-1, m,n)];
	    gn=KK*gn+ar[inx2(i0-1, i1,   m,n)];
	    gn=KK*gn+ar[inx2(i0,   i1+1, m,n)];
	    gn+=  GH*ar[inx2(i0,   i1,   m,n)];
  br[inx2(i0,i1,m,n)]=gn;
  }

/* phase to update cells */
for (i0=0; i0<KK; i0++)    lf22[i0] =0;
for (i0=0; i0<TS; i0++)   ttus[i0] =0;
for (i0=0; i0<DS; i0++)   rus22[i0] =0;
for (i0=0; i0<2*NG; i0++) gus22[i0] =0;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
  gn=br[inx2(i0,i1,m,n)];
  ar[inx2(i0,i1,m,n)]=sqrule22[gn]-'0';
  lf22[ar[inx2(i0,i1,m,n)]]+=1;
//  gus22[gotb[gn]]+=1;
  ttus[tc[gn]]+=1;
  rus22[gn]++;
  }

/* phase to show results */
}	 /* end aronegen */
    
/*find fraction of live cells in arry[m][n] */
void arrstatecount(aa,m,n) char *aa; int m, n; {int i0, i1;
  for (i0=0; i0<KK; i0++) lf22[i0]=0;
  for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) 
	lf22[aa[inx2(i0,i1,m,n)]]+=1;
}

int inx2(i,j,m,n) int i, j, m, n; {return((n+j)%n+n*((m+i)%m));}

/* display a screen of evolution */
void arrscreen(aa,m,n) char *aa; int m, n; {int i, j;
  for (j=0; j<m; j++) for (i=0; i<n; i++) videodot(j,i,aa[n*j+i]);}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
/* l     = color of fiducial marks      */
void arrgfram(n,l) int n, l; {int i;
for (i=0; i<=n; i++) 
	{arrbdot(i,0,0);arrbdot(i,n,0);arrbdot(n,i,0);arrbdot(0,i,0);};
for (i=0; i<=2*n; i+=15) 
	{arrbdot(i/2,0,l);arrbdot(i/2,n,l);arrbdot(n,i/2,l);arrbdot(0,i/2,l);};
}

/* put a diagonal in the fixpoint graph */
void arrgdiag(n,l) int n, l; {int i; for (i=0; 2*i<n; i++) arrbdot(2*i,2*i,l);}

/* graph Bernstein polynomial */
void barrgraf(k,n) int k, n; {int i; double arrburn(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  arrbdot(i,(int)(en*arrburn(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double arrburn(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* arrbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
void arrbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* arrbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
void arrbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* display Wolfram rule number of planar cross section */
/* calculate LCAU22 wru from a VN3D22 argument rule    */
void wxsscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i1][i1][i1][i1][i2][i1];
  wrru22(c,ur);
  }

/* display Wolfram rule number of diagonal cross section */
/* calculate LCAU22 wru from a VN3D22 argument rule    */
void wysscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i0][i0][i2][i2][i2][i1];
  wrru22(c,ur);
  }

/* calculate wru from an argument rule	*/
void wrru22(c,ru) char *c, ru[KK][KK][KK]; {int  k, i0, i1, i2;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
    k=2*k+(ru[i0][i1][i2]-'0');
  sprintf(c,"%3d",k);
}

/* end VN2DView.m */
