
/* prob26.c						*/
/* program for LCAU26 option 't'			*/
/* calculate probabilities related to evolution		*/
/* Harold V. McIntosh, 10 August 1987			*/
/* 29 February 1988 - adapted from LCAU21 to LCAU22	*/
/* 11 March 1989 - adapted from LCAU23 to LCAU24	*/
/* 16 March 1989 - draw diagonal in B panel		*/
/* 18 March 1989 - Options n,N - totalistic statistics	*/
/* 15 September 1989 - entropy calculations		*/
/* 22 December 1991 - Bernstein curve fitting etc.	*/
/* 20 January 1992 - separate out prob2r, probk5.	*/
/* 10 February 1992 - Houston Instrument plotter option	*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*  Copyright (C) 1987  */
/*  Copyright (C) 1988  */
/*  Copyright (C) 1989  */
/*  Copyright (C) 1992  */
/*  Harold V. McIntosh  */

# include <math.h>
# include <stdio.h>
# include "rec.h"
# include "nxlcau26.h"
# include "keybrd.h"

extern char   ttrule[TS];
extern char   strule[KK][TS-1];
extern char   cstr[CLEN];				/* console rec program */
extern char   recrule[][48];
// extern char   drectabl[][HCH];
extern char   ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern int    binrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char   arr1[AL];
extern int    rij, rdi, rdj;				/* rec demo pointers */
extern int    co, dn;					/* connectivity, dimension */
extern int    cp;					/* critical point          */
extern int    pr, pl;					/* rule, line prob, mils   */
extern int    bi;					/* bernstein index	   */
extern int    ct0, ct1, ct2, ct3;			/* general counters	   */
extern int    sw0, sw1, sw2, sw3;			/* general switches	   */
extern int    ex, wy;					/* hiplot coordinates	   */
extern double nu0, nu1, nu2, nu3;			/* general numeric values  */
extern double mf;					/* mouse increment	   */
extern double sm[TS];					/* selfconsistent nbrhd    */
extern double sn[TS];					/* selfconsistent nbrhd    */
extern double bx[BD];					/* bernstein factor        */
extern double bp[BD];
extern double pv[10];					/* polynomial vector 	   */
extern double fpa, fpb, fpc;				/* fixed points		   */

extern Inst   rprg[];					/* rec program array */
extern struct fptbl dtbl[];

extern long int r_intpar;
extern double   r_dblpar;

# define GROW	   12	/* row for graph panels		*/
# define BROW	    3	/* row for bar charts		*/
# define EROW	    1	/* row for evolution synopsis	*/
# define AORG 	    0	/* x-origin  of contour plot    */
# define BORG 	  109	/* x-origin  of 2-block param   */
# define CORG 	  219	/* x-origin  of Bernstein plot  */

# define stdaux stderr

# include "nxprob2r.c"
# include "nxprobk6.c"

/* ---------------------------------------------------- */
/* -            edit the probability screen           - */
/* ---------------------------------------------------- */

edtri() {char c; int i; double smq();

  videomode(COLGRAF);
  videopalette(YELREGR);
  prmenu();

  while (0<1) {
  runu();
  videocursor(0,0,36); videoputc('?',2);
  c=kbdin();
  if (c == '\015') break;
  videocursor(0,0,38); videoputc(c,2);
  videocursor(0,0,36); videoputc(' ',0);
  switch (c) {


    case Kf4: if (rec_c(cstr,rprg,PLEN,dtbl)==1) rec_x(rprg); break;
    case Kf5: if (rec_c(&recrule[rij],rprg,PLEN,dtbl)==1) rec_x(rprg); break;

    case KF4: clep('e'); videocursor(0,EROW,0); printf("%0.40s",cstr); break;
    case KF5: clep('e'); videocursor(0,EROW,0); printf("%0.40s",&recrule[rij]); break;



/*  case INS: twodi(dij); break;					/* INSERT key */
/*
/*  case DEL: videoclear(); break;				/* DELETE key */
/*
/*  case PGU: dij=(DROWS+dij-1)%DROWS; twodi(dij); break;		/* PAGE UP key */
/*
/*  case PGD: dij=(DROWS+dij+1)%DROWS; twodi(dij); break;		/* PAGE DOWN key */
/*
/*  case KGU: dij=1; twodi(dij); break; 				/* ^PAGE UP key */
/*
/*  case KGD: dij=DROWS; twodi(dij); break;			/* ^PAGE DOWN key */

    case PPU: rij=(RROWS+rij-1)%RROWS; recdi(rij); break;	/* PAR UP key	 */
    case PPD: rij=(RROWS+rij+1)%RROWS; recdi(rij); break;	/* PAR DOWN key  */
    case KPU: rij=0; recdi(rij); break;				/* ^PAR UP key	 */
    case KPD: rij=RROWS-1; recdi(rij); break;			/* ^PAR DOWN key */

    case '+': videopalette(WHCYMAG); break;
    case '-': videopalette(YELREGR); break;
//    case 'a': asfreq(3); break;
    case 'A': psevolve(); break;
    case 'b': rprob(pl); break;
    case 'B': pl=numin(0); prmenu(); break;
    case 'c': traj(numin(0),8,3); break;
    case 'C': trajy(numin(0),0); break;
    case 'd': trajy(numin(0),2500); break;
    case 'D': trajy(numin(0),50000); break;
    case 'e': pevolve(); break;
    case 'E': plevolve(); break;
    case 'f': ranchay(ascrule,pr,DS); break;
    case 'F': pr=numin(0); prmenu(); break;
    case 'g': lifreq(50,1,2); break;
    case 'G': lifreq(200,1,1); break;
    case '.': lifreq(1,1,2); break;
    case 'h': egraf(CORG,0,100); break;
    case 'i': piter(100,2,3); break;
    case 'j': piter(100,3,3); break;
    case 'J': piter(100,numin(0),3); break;
    case 'k': piter(100,4,3); break;
    case 'l': piter(100,5,3); break;
    case 'L': lesq(50,2); break;
    case 'H': lesq(3,2); break;
    case 'm': moncar(1,2); break;
    case 'M': moncar(50,1); break;
    case 'n': clep('m'); neisum(AL,25); break;
    case 'N': clep('m'); nbrsum(AL,10); break;
    case 'o': co=numin(0); prmenu(); break;
    case 'O': dn=numin(0); prmenu(); break;
    case 'p': ptseq(100); break;
    case 'P': hipro(100,0); break;
    case 'r': ranmf(100,numin(0),0,2); break;
    case 'R': ranmf(200,numin(0),1,2); break;
    case 's': lifreq(50,3,2); break;
    case 'S': lifreq(50,2,2); break;
    case 'X': bx[bi]=smq(); prmenu(); break;
    case 'I': bi=numin(0); prmenu(); break;
    case 'x': pdiff(100); break;
    case 'y': pdiff3(100); break;
    case 'w': pdiff4(100); break;
    case 'v': inlay(4,4,3); inlay(6,2,8); break;
    case 'V': cubch(12); break;
    case 't': twoblockfreq(100); break;
    case '1': clep('m'); oneblfreq(300,48); break;
    case '2': clep('m'); twoblfreq(300,48); break;
    case '3': clep('m'); thrblfreq(300,48); break;
    case '4': clep('m'); foublfreq(300,48); break;
    case '5': clep('m'); fivblfreq(300,48); break;
    case '6': clep('m'); sixblfreq(300,48); break;
    case '7': clep('m'); svnblfreq(300,48); break;
    case '?': trmenu(); break;
    case '!': prmenu(); break;
    case 'z': clep(kbdin()); break;
    case '/': videomode(COLGRAF); videopalette(YELREGR); break;
    case '*': bcprin(); break;
    case '#': parfix(); break;
    case '&': sctf(100,smq()); break;
    case '$': for (i=0; i<11; i++) sctf(100,0.0999*((double)i)); break;
    case '%': clep('m'); neidif(8*BROW); break;
    case 'K': neicon(); break;

    case '=': aijpanel(ct1/3,ct1%3,ct3++,3); break;
    case '|': while (!kbdst()) {onegen(AL); scanaij(1,1,1);} kbdin(); break;
    case '~': loclhden(3); break;

    case '@': sw0=numin(0); break;
    case '^': hiplot00(); break;
    case 'Z': hiplotxx(); break;

    case MOD: gbmar(bi,1); if (bi>0) bi--; gbmar(bi,2); prmenu(); break;
    case MOU: gbmar(bi,1); if (bi<TS-1) bi++; gbmar(bi,2); prmenu(); break;
    case MOR: bx[bi]+=mf; prmenu(); break;
    case MOL: bx[bi]-=mf; prmenu(); break;
    case MOA: mf*=0.5; prmenu(); break;
    case MOB: mf*=2.0; prmenu(); break;
    case MOX: for (i=0; i<BD; i++) bx[i]=1.0; mf=0.05; prmenu(); break;

    default: break;
    } /* end switch */
  }   /* end while  */
  videopalette(WHCYMAG);
  videomode(T80X25);
}      /* end edtri  */


/* show probability curves */
void procur(e,g) int e, g; {
  switch (e) {
    case 'e': switch (g) {
      case 0: gfram(100); gdiag(100); break;
      case 1: pdiff(100); break;
      case 2: pdiff3(100); break;
      case 3: pdiff4(100); break;
//      case 4: pdiff5(100); break;
//      case 5: pdiff6(100); break;
      default: break;} break;
    case 'i': switch (g) {
      case 0: gfram(100); gdiag(100); break;
      case 1: pdiff(100); break;
      case 2: pidiff(100); break;
      case 3: pjdiff(100); break;
      case 4: pkdiff(100); break;
      case 5: pldiff(100); break;
      default: break;} break;
    default: break;}
}

/* show n-block probabilities */
void nblcur(n) int n; {switch (n) {
    case '1': oneblfreq(300,48); break;
    case '2': twoblfreq(300,48); break;
    case '3': thrblfreq(300,48); break;
    case '4': foublfreq(300,48); break;
    case '5': fivblfreq(300,48); break;
    case '6': sixblfreq(300,48); break;
    case '7': svnblfreq(300,48); break;
    default: break;}
}

/* option 'n' (general neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* mm - number of cells	*/
/* nn - number of lines	*/

void gnlsum(mm,nn) int mm, nn; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
int    ii, k, m, n, l;
int    b[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
double em, en, f;
double bf[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
double bft[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++) 
for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++) 
for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) 
	bft[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  gnrsum(mm,b);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
  for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++) 
  for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++) 
  for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) 
	bf[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia] = 
		((double)(b[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]));
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
  for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++) 
  for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++) 
  for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) 
	bft[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia] += 
		bf[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia];
  m=0; videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++) 
for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) {
    n=(int)(f*bf[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i2%2+1);
    }
  videodot(ii,m++,3);
  k++;
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(nn,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) {
    n=(int)(0.1*(bft[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]/en));
    if (n>0) for (l=0; l<n; l++) videodot(nn,m++,i0%2+1);
    }
  videodot(nn,m++,3);
}

/* gather general neighborhood sums for one generation		*/
/* mm = number of cells        			       		*/
/* b[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK]  = sum counts 	*/
void gnrsum(mm,b) int mm, b[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK]; {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, i;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
  for (i3=0; i3<KK; i3++) for (i4=0; i4<KK; i4++) for (i5=0; i5<KK; i5++) 
  for (i6=0; i6<KK; i6++) for (i7=0; i7<KK; i7++) for (i8=0; i8<KK; i8++) 
  for (i9=0; i9<KK; i9++) for (ia=0; ia<KK; ia++) 
	 b[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]=0;

 b[arr1[mm-10]][arr1[mm-9]][arr1[mm-8]][arr1[mm-7]][arr1[mm-6]][arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]]++;
  b[arr1[mm-9]][arr1[mm-8]][arr1[mm-7]][arr1[mm-6]][arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]]++;
  b[arr1[mm-8]][arr1[mm-7]][arr1[mm-6]][arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]]++;
  b[arr1[mm-7]][arr1[mm-6]][arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]]++;
  b[arr1[mm-6]][arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]]++;
  b[arr1[mm-5]][arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[5]]++;
  b[arr1[mm-4]][arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[5]][arr1[6]]++;
  b[arr1[mm-3]][arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[5]][arr1[6]][arr1[7]]++;
  b[arr1[mm-2]][arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[5]][arr1[6]][arr1[7]][arr1[8]]++;
  b[arr1[mm-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[5]][arr1[6]][arr1[7]][arr1[8]][arr1[9]]++;
for (i=0; i<mm-10; i++) 
  b[arr1[i]][arr1[i+1]][arr1[i+2]][arr1[i+3]][arr1[i+4]][arr1[i+5]][arr1[i+6]][arr1[i+7]][arr1[i+8]][arr1[i+9]][arr1[i+10]]++;
 }

/* ----------------------------------------------------------------- */

/* ----------------------------------- */
/* rec variants of the menu operations */
/* ----------------------------------- */
/* -------------------------------------------*/
/* definition of REC operators and predicates */
/* -------------------------------------------*/

rec(z) char *z; {if (rec_c(z,rprg,PLEN,dtbl)==1) rec_x(rprg);}

int kbint() {if (kbdst()) {kbdin(); return 0;} else return 1;}
int dpred() {if (rand()%1000>100) return 0; else return 1;}
int rpred() {if (rand()%1000>499) return 0; else return 1;}

int tilde() {prmenu();}

ropla() {} /* {asfreq(3);} */
roplb() {rprob(pl);}
ropld() {diagl(*r_pc++);}
rople() {loclhden(1+ct2%3);}
roplf() {frame(*r_pc++);}
roplg() {retgen(2);}
ropli() {piter(100,nibbl(*r_pc++),3);}
ropln() {clep('m'); neisum(AL,1);}
roplo() {co=nibbl(*r_pc++);}
roplp() {videodot(199-(int)(nu1),BORG+ct2,1+ct2%3); ct2=(100+ct2+1)%100;}
roplr() {ranmf(100,*r_pc++,0,2);}
roplt() {twoblockfreq(100);}
roplx() {engen(*r_pc++);}
roplz() {clep(*r_pc++);}

ropua() {psstrip(ct0); ct0=(ct0+1)%3;}
ropub() {pl=nibbl(*r_pc++);}
ropud() {scanaij(ct1/3,ct1%3,1+ct2%3); ct1=(ct1+1)%9;}
ropue() {aijpanel(ct1/3,ct1%3,3,1+ct2%3); ct1=(ct1+1)%9;}
ropug() {retgen(1+ct2%3);}
ropuk() {neicon();}
ropuo() {dn=nibbl(*r_pc++);}
ropuq() {cubch(8);}
ropur() {ranmf(100,*r_pc++,1,2);}
roput() {nutoto((int)r_dblpar); totoasc(); runu();}

/*
ropld() {onedot(nibbl(*r_pc++),128,128);}
*/

/* identify the current REC program (in panel E) */
recdi(n) int n; {
// clep('e');
// videocursor(0,EROW,0); printf(" f4 executes ed. program; f5, demo"); 
// videocursor(0,EROW+1,0); printf("%0.40s",&drectabl[n]);
}

/* ----------------------------------------------------------------- */

/* calculate and display local densities */

loclhden(l) int l; {int i0, i1, i2, i, j, k, s, t[2][2][1];
  for (i0=0; i0<2; i0++)
  for (i1=0; i1<2; i1++)
  for (i2=0; i2<1; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=16)
  for (i1=0; i1<32; i1+=16)
  for (i2=0; i2<16; i2+=16) {
    s=0;
    for (i=0; i<16; i++)
    for (j=0; j<16; j++)
    for (k=0; k<16; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/16][i1/16][i2/16]+=s;
    }

  for (i0=0; i0<2; i0++)
  for (i1=0; i1<2; i1++)
  for (i2=0; i2<1; i2++) {
    s=i0+2*(i1+2*i2);
    videodot(t[i0][i1][i2]/40,5*s,l);
    }
}

/* calculate and display local densities */

locl8den(l) int l; {int i0, i1, i2, i, j, k, s, t[4][4][2];
  for (i0=0; i0<4; i0++)
  for (i1=0; i1<4; i1++)
  for (i2=0; i2<2; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=8)
  for (i1=0; i1<32; i1+=8)
  for (i2=0; i2<16; i2+=8) {
    s=0;
    for (i=0; i<8; i++)
    for (j=0; j<8; j++)
    for (k=0; k<8; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/8][i1/8][i2/8]+=s;
    }

  for (i0=0; i0<4; i0++)
  for (i1=0; i1<4; i1++)
  for (i2=0; i2<2; i2++) {
    s=i0+4*(i1+4*i2);
    videodot(t[i0][i1][i2]/5,3*s,l);
    }
}

/* calculate and display local densities */

locl4den(l) int l; {int i0, i1, i2, i, j, k, s, t[8][8][4];
  for (i0=0; i0<8; i0++)
  for (i1=0; i1<8; i1++)
  for (i2=0; i2<4; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=4)
  for (i1=0; i1<32; i1+=4)
  for (i2=0; i2<16; i2+=4) {
    s=0;
    for (i=0; i<4; i++)
    for (j=0; j<4; j++)
    for (k=0; k<4; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/4][i1/4][i2/4]+=s;
    }

  for (i0=0; i0<8; i0++)
  for (i1=0; i1<8; i1++)
  for (i2=0; i2<4; i2++) {
    s=i0+8*(i1+8*i2);
    videodot((int)((100.0*(double)t[i0][i1][i2])/64.0),s%100,l);
    }
}

/* ----------------------------------------------------------------- */

/* initialize some layers for the 3-d automaton */

inlay(i,j,k) int i, j, k; {int i0, i1;
  for (i0=0; i0<25; i0++)
  for (i1=0; i1<25; i1++) {
    arr1[ix3(i+i0,j+i1,k-1)]=0;
    arr1[ix3(i+i0,j+i1,k)]=1;
    arr1[ix3(i+i0,j+i1,k+1)]=0;
    }
}

/* make a cubical checkerboard */
cubch(n) int n; {int i0, i1, i2;
if (n==0) return;
  for (i0=0; i0<32; i0++)
  for (i1=0; i1<32; i1++)
  for (i2=0; i2<16; i2++)
    arr1[ix3(i0,i1,i2)]=(i0/n+i1/n+i2/n)%2;
}

/* -------------------------------------- */
/* show the evolution of a planar section */
/* -------------------------------------- */

plevolve() {int i, j, l, i0, i1, i2;
clep('a');
gfram(100);
asctobin();
for (i=0; i<3; i++) for (j=0; j<3; j++) 
  {aijpanel(i,j,3,(i+j)%2+1); onegen(AL);}
}

/* show single ij subpanel of panel A */

aijpanel(i,j,k,l) int i, j, k, l; {int i0, i1, i2, ll;
  switch (dn) {
    case 2: case 5: case 7:
      for (i0=0; i0<32; i0++)
      for (i1=0; i1<32; i1++) {
        ll=arr1[ix2(i0,i1)];
        videodot(i1-33*j,i0+33*i,ll==0?0:l);
        }
      break;
    case 3: default:
      i2=k;
      for (i0=0; i0<32; i0++)
      for (i1=0; i1<32; i1++) {
        ll=arr1[ix3(i0,i1,i2)];
        videodot(i1-33*j,i0+33*i,ll==0?0:l);
        }
      break;}
}

/* ------------------------------------------- */
/* show the evolution of three planar sections */
/* ------------------------------------------- */

psevolve() {int i, l, i0, i1, i2;
  clep('a');
  gfram(100);
  asctobin();
  for (i=0; i<3; i++) {
    aijpanel(i,0,3,i%2+1);
    aijpanel(i,1,4,i%2+2);
    aijpanel(i,2,5,i%2+1);
    }
  onegen(AL);
}

/* show a single strip of psevolve for use in a rec program */

psstrip(i) int i; {
aijpanel(i,0,3,i%2+1); aijpanel(i,1,4,i%2+2); aijpanel(i,2,5,i%2+1);}

/* run through all 16 slices of 32x32x16 cube */
scanaij(i,j,l) int i, j, l; {int k; for (k=0; k<16; k++) aijpanel(i,j,k,l);}

/* -------------------------------------- */
/* show the evolution of a planar section */
/* -------------------------------------- */

pplevolve() {int i, j, l, i0, i1, i2, i3, i4;
  clep('a');
  gfram(100);
  asctobin();
  for (i=0; i<8; i++) for (j=0; j<8; j++) {
          i1=i3=i4=0;
	  for (i0=0; i0<8; i0++)
	  for (i2=0; i2<8; i2++)
    {
    l=arr1[ix5(i0,i1,i2,i3,i4)];
    videodot(i2-12*j,i0+12*i,l+1);
    }
    onegen(AL);
  }
}

/* evaluate the nth generation Bernstein polynomial at point p */

double bern(p,n) double p; int n; {
int    i, j, d;
double q, s, pq[BD];
  d=12*n-11;
  q=1.0-p;
  for (i=0; i<=d; i++) pq[i]=1.0;
  for (i=1; i<=d; i++) {
    for (j=0; j<i; j++) pq[j]*=q;
    for (j=i; j<=d; j++) pq[j]*=p;
    }
  for (i=0, s=0.0; i<=d; i++) s+=bp[i]*pq[i];
  return(s);
}

/* evaluate the parameters for two-block probabilities after one generation */

twoblock(z) double *z; {
int    i, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib;
int    j, j0, j1;
double w, x, y, p, q, pp;
double a[KK][KK], b[KK], c[KK][KK];

p=z[0]; q=1.0-p; pp=z[1];
a[1][1]=pp; a[1][0]=a[0][1]=p-pp; a[0][0]=1.0-2*p+pp;
b[1]=p; b[0]=q;
for (i=0; i<KK; i++) {for (j=0; j<KK; j++) c[i][j]=0.0;}
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]-'0';
  j1=ascrule[i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib]-'0';
  x=a[i0][i1]*a[i1][i2]*a[i2][i3]*a[i3][i4]*a[i4][i5]*a[i5][i6]
	*a[i6][i7]*a[i7][i8]*a[i8][i9]*a[i9][ia]*a[ia][ib];
  y=b[i1]*b[i2]*b[i3]*b[i4]*b[i5]*b[i6]*b[i7]*b[i8]*b[i9]*b[ia];
  w=0.0; if (y>0.0001) w=x/y;
  c[j0][j1]+=w;
  }
z[0]=c[1][0]+c[1][1]; z[1]=c[1][1];
}

/* graph the 2-block parameter differences to find self-consistent values */
/* nxn points 								  */

twoblockfreq(n) int n; {
int    i, j, l;
double op[2], np[2], s, t, u, v;

gfram(100);
printf("hello from twoblockfreq\n");
s=1.0/((double)(n));
for (op[1]=s, i=1; i<n; op[1]+=s, i++) {
  for (op[0]=s, j=1; j<n; op[0]+=s, j++) {
    np[0]=op[0]; np[1]=op[1]; twoblock(np);
    u=np[0]-op[0]; v=np[1]-op[1];
    t=u*u+v*v;
    if (t<0.0025) l=0; else {
    if (t<0.0675) {if((i+j)%2) l=0; else l=3;} else {
    if (t<0.3906) l=3; else {
    if (t<0.7500) {if((i+j)%2) l=3; else l=2;} else {
    if (t<1.5000) l=2; else {
    if (t<3.0000) {if((i+j)%2) l=2; else l=1;} else {
    if (t<6.0000) l=1; else {
                  if((i+j)%2) l=1; else l=0;
       }}}}}}}
    videodot(i,j,l);
    }  /* end for-j */
  }    /* end for-i */
}

/* --------------------------------------- */
/* some programs that belong mostly to rec */
/* --------------------------------------- */

/* show the rule number in panel R */

runu() {
  videocursor(0,0,0); videoputc(':',2);
  if (istot()) {scrstr("  T:"); tscrrul(); scrstr("    LCAU26");}
    else if (isstot()) {scrstr("  S:");
	sscrrul(0); scrstr(",");
	sscrrul(1); scrstr("    LCAU26");}
    else scrrul();
}

/* show a frame */
frame(c) char c; {switch (c) {
  case 'a': gfram(100); break;
  case 'b': gfram(100); break;
  case 'c': gfram(100); break;
  case 'H': hiplot(20,80,0); hiplot(20,180,1); hiplot(120,180,1);
		hiplot(120,80,1); hiplot(20,80,1); break;
  default: break;}}

/* show a diagonal */
diagl(c) char c; {switch (c) {
  case 'a': gdiag(100); break;
  case 'b': gdiag(100); break;
  case 'c': gdiag(100); break;
  case 'H': hiplot(20,80,0); hiplot(120,180,1); hiplot(20,80,0); break;
  default: break;}}

/* graph mean field for nth generation */
engen(c) char c; {switch (c) {
  case '2': pdiff(100);  if (sw0>0) hipbgraf(2); break;
  case '3': pdiff3(100); if (sw0>0) hipbgraf(3);  break;
  case '4': pdiff4(100); if (sw0>0) hipbgraf(4);  break;
  default: break;}}

/* graph Bernstein polynomial on the hiplot plotter */

hipbgraf(k) int k; {int i; double bern(), p;
for (i=0,p=0.0; i<100; i++,p+=0.01) {hiplot(20+i,80+(int)(100.0*bern(p,k)),1);}}

/* display state frequencies in arr1 as a point on probability graph */
retgen(l) int l; {int j, os, ns; double of, nf, s;
s=100.0/((double)AL);
asctobin();
for (j=0, os=0; j<AL; j++) if (arr1[j]==1) os++;
onegen(AL);
for (j=0, ns=0; j<AL; j++) if (arr1[j]==1) ns++;
nu0=of=s*((double)os); nu1=nf=s*((double)ns);
videodot((int)(of),(int)(nf),l);
}

/* scan the mean field curve using random cell densities */
ranmf(n,g,h,l) int n, g, h, l; {
int     i, j, pp, dp, os, ns;
double of, nf, s;
dp=1000/n;
s=100.0/((double)AL);
asctobin();
for (pp=0, i=0; i<n; pp+=dp, i++) {
  rprob(pp);
  if (g>0) for (j=0; j<g; j++) onegen(AL);
  for (j=0, os=0; j<AL; j++) if (arr1[j]==1) os++;
  onegen(AL);
  for (j=0, ns=0; j<AL; j++) if (arr1[j]==1) ns++;
  of=s*((double)os); nf=s*((double)ns);
  if (h==0) {videodot((int)(0.1*pp),(int)(nf),l);
             if (sw0==1) hiplot(20+(int)(0.1*pp),80+(int)nf,(i==0)?0:1);}
  if (h==1) {videodot((int)(of),(int)(nf),l);
             if (sw0==1) hiplot(20+(int)of,80+(int)nf,(i==0)?0:1);}
  }
}

/* graph the iterated probability */
/* n = number of points (100)	  */
/* m = number of iterations	  */
/* l = color of dot		  */

hipiter(n) int n; {int i, j; double bern(), en, p, q, r;
en=0.001;
berncoef(bx);
hifplot(200,800,0);
for (p=0.0, i=0; i<=1000; p+=en, i++) {
  for (q=p, j=0; j<n; j++) {r=q; q=bern(r,2);}
  hifplot(200+(int)(1000.0*p),800+(int)(1000.0*q),1);
  }
hifplot(200,800,0);
}

/* hiplot subroutines */

/* show the rule number in panel R */

hirunu() {int i;
  hiplot(10,186,0);
  if (istot()) {fprintf(stdaux,"S12+T:");
     for (i=0; i<TS; i++) fprintf(stdaux,"%c",ttrule[i]);
     fprintf(stdaux,"          LCAU26");
     fprintf(stdaux,"_");}
    else if (isstot()) {fprintf(stdaux,"S12+S:");
     for (i=0; i<TS-1; i++) fprintf(stdaux,"%c",strule[0][i]);
     fprintf(stdaux,", ");
     for (i=0; i<TS-1; i++) fprintf(stdaux,"%c",strule[1][i]);
     fprintf(stdaux,"._");}
    else {};
}

hidate() {long cl[1]; char *ctime(); time(cl);
  hiplot(60,10,0); fprintf(stdaux,"S11+LCAU26_");
  hiplot(60,10,0); fprintf(stdaux,"S11+LCAU26  -  %26s _",ctime(cl));}

hiplot(x,y,l) int x, y, l; {switch (l) {
case 0: fprintf(stdaux,"U %4d,%4d ",10*x,10*y); break;
case 1: fprintf(stdaux,"D %4d,%4d ",10*x,10*y); break;
default: break;}}

hifplot(x,y,l) int x, y, l; {switch (l) {
case 0: fprintf(stdaux,"U %4d,%4d ",x,y); break;
case 1: fprintf(stdaux,"D %4d,%4d ",x,y); break;
default: break;}}

hiplot00() {fprintf(stdaux,";: P1 EC5 V1 A "); hidate(); hirunu();}
hiplotxx() {fprintf(stdaux,"P0 @ ");}

/* end prob26.c */
