
/* nxrijn26.c						*/
/* program for lcau26.c option 'd'			*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 3 March 1988 - adapted from lca21.c to lca22.c	*/
/* 12 March 1989 - adapted from lca22.c to lca23.c	*/
/* 5 April 1989 - promoted further to lcau24.c		*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl24.c incorporated into rijn24.c	*/
/* 20 November 1989 - loop in kwait discarded		*/
/* 30 July 1994 - adapted to NeXT			*/

/*  Copyright (C) 1987	*/
/*  Copyright (C) 1988	*/
/*  Copyright (C) 1989	*/
/*  Copyright (C) 1994	*/
/*  Copyright (C) 1995	*/
/*  Harold V. McIntosh	*/

# include <math.h>
# include "nxlcau26.h"
# include "nxrijn26.h"
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>

extern NXColor colst[KK];

# define JX  0.50  /* x-center, de Bruijn diagram	*/
# define JY  0.50  /* y-center, de Bruijn diagram	*/
# define RI 0.400  /* inner radius, de Bruijn diagram	*/
# define RO 0.425  /* outer radius, de Bruijn diagram	*/

extern char arr1[];
extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern int  binrule[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

int    dd;		/* # points in de Bruijn ring		*/
int    dbno;		/* node number in de Bruijn diagram 	*/
double dcntr[XDB][2];

char   aarry[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

/* edit the de Bruijn diagram */
/* g = number of generation (0==2nd)	*/
/* s = shift distance (e==0)		*/
/* r = show transients 			*/

void edijn(g,s,p,r) int g, s, p, r; {
  switch (g+'a') {

    case 'A': xpass1(s,r); break;

    case 'a': switch (s+'a') {

      case 'a': apass1(0,p,r);  break;
      case 'b': apass1(1,p,r);  break;
      case 'c': apass1(2,p,r);  break;
      case 'd': apass1(3,p,r);  break;
      case 'e': apass1(4,p,r);  break;
      case 'f': apass1(5,p,r);  break;
      case 'g': apass1(6,p,r);  break;
      case 'h': apass1(7,p,r);  break;
      case 'i': apass1(8,p,r);  break;
      case 'j': apass1(9,p,r);  break;
      case 'k': apass1(10,p,r); break;
      default: break;} break;
    default: break;
    }
}

/* clear edijn screen and set up menu */
void clijn() {
  videomode(COLGRAF);
  videopalette(YELREGR);
  videocursor(0,0,0);
  scrrul();
  videocursor(0,3,0);
  printf("de Bruijn ring\n\n");
  printf("ab   -  0*,1*\n");
  printf("AB   -  full n*\n");
  printf("cdefghi - (1,x)\n");
  printf("CDEFG - (all)\n");
  printf("h.l.p - (2,x)\n");
  printf("y - cycles 1-9 \n");
  printf("1234 - diagram\n");
  printf("+-   - pallette\n");
  printf("?/   - clr scrn\n");
  printf("<cr> - exit\n\n");
}

/* edit the Pass 6 shift displays */
void edshg(g,s) int g, s; {
    switch (g) {
      case 0: apass6(); break;
    default: break;}
}

/* edit the Pass 6 constant displays */
void edcst(g) int g; {
    switch (g) {
      case 0: apass6(); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 shift displays */
/* pass5 fills the initial line of evolution */
/* shflev decides which connectivity matrix  */
void edshf(g,s,c) int g, s, c; {
    switch (g) {
    case 0: apass5(c); break;
    default: break;}
}

/* edit the Pass 5 (sample loops) constant displays */
void edshc(g,c) int g, c; {
  switch (g) {
    case 0: apass5(c); break;
    default: break;}
}

/* find the level of de Bruijn diagram appropriate to g, s  */
int shflev(g,s) int g, s; {int  i; i=s<0?-s:s; return (i-1)/5;}

int permut(k,n) int k, n; {
  switch (n) {
    case 0: break;
    case 1: k=(k+1)%KK; break;
    default: break;}
  return k;
}

/* increment ix */ 
void incix(ix,n) int *ix, n; {int i; 
  for (i=0; i<n; i++) {
    ix[i]++;
    if (ix[i]>=KK) ix[i]=0; else return;
    }
}

/* selected node in ix */ 
void nodix(ix,n) int *ix, n; {int i, j; 
  j=dbno;
  for (i=0; i<n; i++) {ix[n-i-1]=j%KK; j/=KK;}
}

/* random number in ix */ 
void ranix(ix,n) int *ix, n; {int i, j; 
  j=random();
  for (i=0; i<n; i++) {ix[i]=j%KK; j/=KK;}
}

/* clear ix */ 
void zerix(ix,n) int *ix, n; {int i; for (i=0; i<n; i++) ix[i]=0;}

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void ijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RI*sin(ii*t+h);
    y=JY-RI*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* insert a link into a de Bruijn diagram */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */

void lijnk(u,v,l) int u, v, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, w, t, sx, sy;

	if (l<KK) NXSetColor(colst[l]); else PSsetgray(NX_BLACK);
	r=10.3;
	w=3.33;
	ex=dcntr[u][0];
	wy=dcntr[u][1];
	x=dcntr[v][0];
	y=dcntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	sx=-w*sin(t);
	sy= w*cos(t);
	eks=x-r*cos(t)+sx;
	wye=y-r*sin(t)+sy;
	PSmoveto(ex+sx,wy+sy);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=dcntr[u][0];
   wy=dcntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",u);
   PSshow(s);

   ex=dcntr[v][0];
   wy=dcntr[v][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",v);
   PSshow(s);
   PSstroke();

}

/* generate a de Bruijn diagram showing ancestors of state k in color l */
void sijn(k,l) int k, l; {
int    i, j, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
  i=((((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9;
  j=((((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9)*KK+ia;
  if (k==ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i0][ia]-'0') lijnk(i,j,l);
  }
}

/* generate a de Bruijn diagram showing the (1,k) links in color l */
void zijn(m,k,l) int m, k, l; {
int    i, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
double th, x, y, t, h;
t=6.28318/((double)(m));
h=0.5*t;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
  i=ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]-'0';
  if ((k==0&&i==i0)||(k==1&&i==i1)||(k==2&&i==i2)||(k==3&&i==i3)||(k==4&&i==i4)) {
    th=((double)(((i0*KK+i1)*KK+i2)*KK+i3)*t+h);
    x=JX-RI*sin(th);
    y=JY-RI*cos(th);
    videoline(x,y,0);
    th=((double)(((i1*KK+i2)*KK+i3)*KK+i4)*t+h);
    x=JX-RO*sin(th);
    y=JY-RO*cos(th);
    videoline(x,y,l);
    }	         /* end if    */
  }              /* end for's */
}	         /* end zijn  */

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videoline(x,y,l) double x, y; int l; {
if (l==0) PSmoveto(400.0*x,400.0*y);
else PSlineto(400.0*x,400.0*y);
}

/* find index of nearest center to x, y (de Bruijn) */
int nearest(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<dd; i++) {
  ex=x-dcntr[i][0];
  wy=y-dcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* ONE26.C */
/* aarry holds 2048 bytes */

/* Pass 1a analyzes all the configurations which fulfil (1,-1+l) */
void apass1(l,p,r) int l, p, r; {
int  ix[RR], m, n;
asctobin();
printf("RIJN26 - apass1: %d, %d, %d\n",l,p,r);
for (ix[0]=0;  ix[0]<KK;  ix[0]++)
for (ix[1]=0;  ix[1]<KK;  ix[1]++)
for (ix[2]=0;  ix[2]<KK;  ix[2]++)
for (ix[3]=0;  ix[3]<KK;  ix[3]++)
for (ix[4]=0;  ix[4]<KK;  ix[4]++)
for (ix[5]=0;  ix[5]<KK;  ix[5]++)
for (ix[6]=0;  ix[6]<KK;  ix[6]++)
for (ix[7]=0;  ix[7]<KK;  ix[7]++)
for (ix[8]=0;  ix[8]<KK;  ix[8]++)
for (ix[9]=0;  ix[9]<KK;  ix[9]++)
for (ix[10]=0; ix[10]<KK; ix[10]++) {
  switch (l) {
    case 0:  m=ix[0];  break;
    case 1:  m=ix[1];  break;
    case 2:  m=ix[2];  break;
    case 3:  m=ix[3];  break;
    case 4:  m=ix[4];  break;
    case 5:  m=ix[5];  break;
    case 6:  m=ix[6];  break;
    case 7:  m=ix[7];  break;
    case 8:  m=ix[8];  break;
    case 9:  m=ix[9];  break;
    case 10: m=ix[10]; break;
    default: break;}
  n=binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]][ix[9]][ix[10]];
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]][ix[9]][ix[10]]=
	n==m?'Y':'N';
  }
if (r) {apass2i(); apass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xpass1(l,r) int l, r; {int  ix[RR], n;
asctobin();
for (ix[0]=0;  ix[0]<KK;  ix[0]++)
for (ix[1]=0;  ix[1]<KK;  ix[1]++)
for (ix[2]=0;  ix[2]<KK;  ix[2]++)
for (ix[3]=0;  ix[3]<KK;  ix[3]++)
for (ix[4]=0;  ix[4]<KK;  ix[4]++)
for (ix[5]=0;  ix[5]<KK;  ix[5]++)
for (ix[6]=0;  ix[6]<KK;  ix[6]++)
for (ix[7]=0;  ix[7]<KK;  ix[7]++)
for (ix[8]=0;  ix[8]<KK;  ix[8]++)
for (ix[9]=0;  ix[9]<KK;  ix[9]++)
for (ix[10]=0; ix[10]<KK; ix[10]++) {
  n=binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]][ix[9]][ix[10]];
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]][ix[9]][ix[10]]=
	n==l?'Y':'N';
  }
if (r) {apass2i(); apass2o();}
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void apass2i() {int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
if ((aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][m]|=0x20;}
} } while (apass3()!=0); }

void apass2o() {int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
if ((aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[m][i0][i1][i2][i3][i4][i5][i6][i7][i8][i9]|=0x20;}
} } while (apass3()!=0); }

/* Pass 3 - erase flags, mark survivors, detect changes */

int apass3() {int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
  switch (aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]) {
    case 'y': aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]='Y'; break;
    case 'Y': aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]='N'; l=1; break;
    case 'n': case 'N': aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 - print loops which remain */
void apass4() {}
/*
void apass4() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8;
int j0, j1, j2, j3, j4, j5, j6, j7, j8;
int k, l, m;
printf(" pass4 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6; j7=i7; j8=i8;
l=0; m=0;
do {
        if (aarry[0][j1][j2][j3][j4][j5][j6][j7][j8]=='Y')
    {aarry[0][j1][j2][j3][j4][j5][j6][j7][j8]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j4=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (aarry[1][j1][j2][j3][j4][j5][j6][j7][j8]=='Y')
    {aarry[1][j1][j2][j3][j4][j5][j6][j7][j8]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) 
	{j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=k;} }}
  } while (l==0);
l=0; m=0;
do {
        if (aarry[j0][j1][j2][j3][j4][j5][j6][j7][0]=='y')
   {aprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   aarry[j0][j1][j2][j3][j4][j5][j6][j7][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][j4][j5][j6][j7][1]=='y')
   {aprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   aarry[j0][j1][j2][j3][j4][j5][j6][j7][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (aarry[j0][j1][j2][j3][j4][j5][j6][j7][0]=='Y')
   {aprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   aarry[j0][j1][j2][j3][j4][j5][j6][j7][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][j4][j5][j6][j7][1]=='Y')
   {aprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   aarry[j0][j1][j2][j3][j4][j5][j6][j7][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=1; m=1;}
  else {l=1; if (m==1) kwait(0);}}
  } while (l==0);
}
}
*/

/* print one link of the de Bruijn diagram */
void aprf(i0,i1,i2,i3,i4,i5,i6,i7,i8) int i0, i1, i2, i3, i4, i5, i6, i7, i8; {
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("%1d",i4);
printf("%1d",i5);
printf("%1d",i6);
printf("%1d",i7);
printf("-");
printf("%1d",i8);
printf("  ");
lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
}

/* Pass 5 - show evolution of sample loops */

void apass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
int j1, j2, j3, j4, j5, j6, j7, j8, j9;
int hl;
int b[KK], irry[RR], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR); break;
    case 'r': ranix(irry,RR); break;
    case 'n': nodix(irry,RR-1);
	      i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
 	      i5=irry[5]; i6=irry[6]; i7=irry[7]; i8=irry[8]; i9=irry[9];
	      for (i=j=0; j<KK; j++)
	        if (aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][j]=='Y') 
		    {b[i]=j; i++;}
	      if (i==0) return;
	      l=0; if (i>1) l=random()%i;
	      irry[RR-1]=b[l];
printf("apass5('n') centered on node %5d\n",dbno);
	      break;
    case 'i': incix(irry,RR); break;
    default:  zerix(irry,RR); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
  i5=irry[5]; i6=irry[6]; i7=irry[7]; i8=irry[8]; i9=irry[9]; ia=irry[10];
for (k=0; k<81; k++) {
  if (aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]=='Y') break;
  incix(irry,RR);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
  i5=irry[5]; i6=irry[6]; i7=irry[7]; i8=irry[8]; i9=irry[9]; ia=irry[10];
  }
hl=YL/2;
arr1[hl-5]=i0;
arr1[hl-4]=i1;
arr1[hl-3]=i2;
arr1[hl-2]=i3;
arr1[hl-1]=i4;
arr1[hl  ]=i5;
arr1[hl+1]=i6;
arr1[hl+2]=i7;
arr1[hl+3]=i8;
arr1[hl+4]=i9;
arr1[hl+5]=ia;
j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6; j7=i7; j8=i8; j9=i9;
for (k=hl-6; k>=0; k--) {
  for (i=j=0; j<KK; j++) if 
	(aarry[j][i0][i1][i2][i3][i4][i5][i6][i7][i8][i9]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
i9=i8; i8=i7; i7=i6; i6=i5; i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5; i6=j6; i7=j7; i8=j8; i9=j9;
for (k=hl+6; k<YL; k++) {
  for (i=j=0; j<KK; j++) if 
	(aarry[i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=i7; i7=i8; i8=i9; i9=ia; ia=arr1[k]=b[l];
  }
}

void apass6() {int i, j, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) 
for (i5=0; i5<KK; i5++) 
for (i6=0; i6<KK; i6++) 
for (i7=0; i7<KK; i7++) 
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
  if(aarry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia]=='Y') {
    i=((((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9;
    j=((((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9)*KK+ia;
    lijnk(i,j,3);
    }
  }
}

/* TW026.C */
/* no space for barry (128K bytes needed) */

/* end nxrijn26.c */
